# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-03-06 15:58
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SocialAccount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('provider', models.CharField(max_length=32)),
                ('uid', models.CharField(max_length=256)),
                ('access_token', models.TextField(blank=True, default='')),
                ('secret', models.TextField(blank=True, default='')),
                ('expires', models.DateTimeField(blank=True, null=True)),
                ('_data', models.TextField(blank=True, default='{}', help_text='Original token response from the OAuth provider.')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='social_accounts', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='socialaccount',
            unique_together=set([('uid', 'provider')]),
        ),
    ]



from __future__ import unicode_literals

from .base import BaseProvider


# access tokens: relatively short-lived, but auto refreshes if set up correctly
class GoogleProvider(BaseProvider):
    type = 'google'

    def __init__(self, **kwargs):
        self.authorization_url = 'https://accounts.google.com/o/oauth2/auth'
        self.access_token_url = 'https://accounts.google.com/o/oauth2/token'
        self.user_api_url = 'https://www.googleapis.com/plus/v1/people/me'
        super(GoogleProvider, self).__init__(**kwargs)

    def connect(self, **kwargs):
        force = kwargs.pop('force', None)
        if force:
            # we need to force the user to re-auth our app (ie. click
            # accept on the Google auth page) - most likely because
            # we need a new refresh_token
            kwargs['approval_prompt'] = 'force'
        return super(GoogleProvider, self).connect(**kwargs)

    def get_social_user_info(self):
        data = super(GoogleProvider, self).get_social_user_info()
        if data:
            uid = data.get('id')
            email = ''
            for tmp in data.get('emails', []):
                if tmp.get('type', '') == 'account':
                    email = tmp.get('email', '')
                    break

            return {
                'source_data': data,
                'uid': uid,
                'username': uid,
                'name': data.get('displayName', ''),
                'first_name': data.get('name', {}).get('givenName', ''),
                'last_name': data.get('name', {}).get('familyName', ''),
                'email': email
            }
        return {}



from __future__ import unicode_literals
import datetime
import hashlib

from django.contrib import messages
from django.contrib.auth import login as auth_login, authenticate, get_user_model
from django.db import IntegrityError
from django.shortcuts import redirect, render
from django.views.generic import View
from six import text_type

try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse

from . import signals
from .forms import PostSocialSignupForm
from .models import SocialAccount
from .providers import registry
from .settings import SIMPLE_SOCIALAUTH_LOGIN_SUCCESS_REDIRECT_URL, SIMPLE_SOCIALAUTH_LOGIN_ERROR_REDIRECT_URL, \
    SIMPLE_SOCIALAUTH_GENERATE_USERNAME


class BaseView(View):
    http_method_names = ['get']

    def dispatch(self, request, *args, **kwargs):
        provider_type = kwargs.get('provider_type', '')
        provider = registry.get(provider_type)

        if not provider:
            return redirect(SIMPLE_SOCIALAUTH_LOGIN_ERROR_REDIRECT_URL)

        kwargs['provider'] = provider

        return super(BaseView, self).dispatch(request, *args, **kwargs)


class LoginView(BaseView):
    def get(self, request, provider_type, provider, *args, **kwargs):
        kw = {}
        if 'force' in request.GET:
            kw['force'] = True

        next_url = request.GET.get('next', '')
        if next_url:
            request.session['simple_socialauth:next_url'] = next_url

        # get the auth url and redirect the user to it to accept/decline
        return redirect(provider.connect(**kw))


class CallbackView(BaseView):
    def get(self, request, provider_type, provider, *args, **kwargs):
        error = request.GET.get('error')
        if error:
            if error == 'access_denied':
                # user clicked cancel on the facebook dialog
                messages.warning(request,
                                 "You declined the {0} authentication request. Shame!".format(provider_type.title()))
            else:
                messages.warning(request,
                                 "Something went wrong with the {0} authentication request. Perhaps try again?".format(
                                     provider_type.title()))

            return redirect(SIMPLE_SOCIALAUTH_LOGIN_ERROR_REDIRECT_URL)

        # process the callback information into a token (the current
        # provider session is auto updated to use the token data)
        try:
            token_data = provider.callback(request.build_absolute_uri())
        except Exception as e:
            # this can fail due to the user denying it; some oauth providers
            # don't pass back the spec-required parameters expected by the
            # oauth lib we use, it blows up - so catch the error and fall back
            # to 'not' authorized.
            token_data = None

        if token_data:
            # List of the possible fields we might get from the various social providers;
            # uid = username = email = name = first_name = last_name = timezone = gender = language = ''

            user_info = provider.get_social_user_info()
            if user_info and 'uid' in user_info:
                user_info['provider'] = provider_type

                # Do we already have a user matching the social provider user id? if so, log in the
                # corresponding Django user.
                try:
                    account = SocialAccount.objects.select_related('user').get(
                        provider=provider_type,
                        uid=user_info.get('uid')
                    )
                except SocialAccount.DoesNotExist:
                    account = None

                if account:
                    if request.user.is_authenticated:
                        if request.user.id != account.user_id:
                            messages.error(request, "This {0} account is associated with another user.".format(
                                provider.display_name()))
                            return redirect(SIMPLE_SOCIALAUTH_LOGIN_ERROR_REDIRECT_URL)
                    else:
                        user = authenticate(pk=account.user_id, social=True)
                        if not user:
                            # `user` will be None if the user is not allowed by `authenticate()` above
                            messages.error(request, "Unable to log in.")
                            return redirect(SIMPLE_SOCIALAUTH_LOGIN_ERROR_REDIRECT_URL)

                        auth_login(request, user)
                elif provider_type == 'google' and 'refresh_token' not in token_data:
                    # special case for google auth - we need to force a re-auth to ensure we get a refresh_token
                    return redirect(reverse('simple_socialauth-login', args=(provider_type,)) + '?force=1')

                if not request.user.is_authenticated:
                    # we need to redirect the user to another page in order to complete
                    # some signup details - save the social user info to the session
                    # for later use
                    request.session['simple_socialauth:user_info'] = user_info
                    request.session['simple_socialauth:token_data'] = token_data
                    return redirect(reverse('simple_socialauth-login_complete', args=(provider_type,)))

                account, created = SocialAccount.create_or_update(request.user, user_info, token_data)

                signals.login.send(
                    SocialAccount,
                    request=request,
                    account=account,
                    user_info=user_info,
                    created=created
                )
                next_url = request.session.pop('simple_socialauth:next_url', '') or SIMPLE_SOCIALAUTH_LOGIN_SUCCESS_REDIRECT_URL
                return redirect(next_url)

        return redirect(SIMPLE_SOCIALAUTH_LOGIN_ERROR_REDIRECT_URL)


class CompleteView(BaseView):
    http_method_names = ['get', 'post']
    form_class = PostSocialSignupForm

    def get(self, *args, **kwargs):
        return self._handler(*args, **kwargs)

    def post(self, *args, **kwargs):
        return self._handler(*args, **kwargs)

    def _handler(self, request, provider_type, provider, *args, **kwargs):
        user_info = request.session.get('simple_socialauth:user_info', {})

        email = user_info.get('email', '').lower().strip()
        username = user_info.get('username', '').strip()

        # we can't legitimately be here if:
        # - we're logged in or
        # - don't have have user_info
        if not user_info or request.user.is_authenticated:
            return redirect(SIMPLE_SOCIALAUTH_LOGIN_ERROR_REDIRECT_URL)

        kw = {
            'username': username,
            'email': email
        }

        if request.method == 'GET':
            # first try to save a new user with their username and email
            # from the social provider; if it fails is_valid() below
            # it probably means data is missing or a matching user already
            # exists for email or username
            form = self.form_class(kw)
        else:
            form = self.form_class(request.POST, initial=kw)

        if form.is_valid():
            user = form.save(commit=False)

            if hasattr(user, 'first_name'):
                user.first_name = user_info.get('first_name') or ''
            if hasattr(user, 'last_name'):
                user.last_name = user_info.get('last_name') or ''
            user.set_unusable_password()

            needs_username = hasattr(user, 'username')
            # loop to allow for race condition induced IntegrityErrors
            # (ie. unique username/email are taken before .save() is
            # called
            while True:
                if needs_username and not user.username and SIMPLE_SOCIALAUTH_GENERATE_USERNAME:
                    h = hashlib.sha256()
                    h.update(text_type(user_info.get('uid', '')).encode('utf8'))
                    h.update(provider_type.encode('utf8'))
                    h.update(email.encode('utf8'))
                    h.update(text_type(datetime.datetime.now()).encode('utf8'))
                    user.username = h.hexdigest()[:user._meta.get_field('username').max_length]

                try:
                    user.save()
                    break
                except IntegrityError:
                    if not needs_username or not user.__class__.objects.filter(username=user.username).exists():
                        # if there isn't a matching user with the same username
                        # already, then it's an IntegrityError for something other
                        # than the the username field - re-raise as we won't handle it.
                        raise
                    user.username = ''

            # clean up the session
            request.session.pop('simple_socialauth:user_info', {})
            token_data = request.session.pop('simple_socialauth:token_data', {})

            user = authenticate(pk=user.pk, social=True)
            auth_login(request, user)

            account, created = SocialAccount.create_or_update(user, user_info, token_data)

            signals.connect.send(
                SocialAccount,
                request=request,
                account=account,
                user_info=user_info,
                created=created
            )

            return redirect(
                request.session.pop('simple_socialauth:next_url', '') or SIMPLE_SOCIALAUTH_LOGIN_SUCCESS_REDIRECT_URL
            )

        context = {
            'form': form
        }

        return render(request, 'simple_socialauth/login_complete.html', context)



from __future__ import unicode_literals

from .base import BaseProvider


class PinterestProvider(BaseProvider):
    type = 'pinterest'

    def __init__(self, **kwargs):
        self.authorization_url = 'https://api.pinterest.com/oauth/'
        self.access_token_url = 'https://api.pinterest.com/v1/oauth/token'
        self.user_api_url = 'https://api.pinterest.com/v1/me/'
        super(PinterestProvider, self).__init__(**kwargs)

    def get_social_user_info(self):
        data = super(PinterestProvider, self).get_social_user_info().get('data')
        if data:
            uid = data.get('id')

            return {
                'source_data': data,
                'uid': uid,
                'username': data.get('username', ''),
                'first_name': data.get('first_name', ''),
                'last_name': data.get('last_name', '')
            }
        return {}



from __future__ import unicode_literals
from django.contrib.sites.models import Site
from requests_oauthlib import OAuth1Session, OAuth2Session

try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse

from ..settings import SIMPLE_SOCIALAUTH_SITEROOT, SIMPLE_SOCIALAUTH_SECURE, SIMPLE_SOCIALAUTH_PROVIDERS_SETTINGS


class BaseProvider(object):
    type = None
    name = None

    authorization_url = None
    request_token_url = None
    access_token_url = None
    user_api_url = None

    @classmethod
    def display_name(cls):
        if cls.name:
            return cls.name
        return cls.type.title()

    def __init__(self, oauth1=False, **kwargs):
        if self.type is None:
            raise Exception('Social provider missing `type` attribute.')

        self.oauth1 = oauth1

        self.settings = SIMPLE_SOCIALAUTH_PROVIDERS_SETTINGS.get(self.type, {})
        if not self.settings:
            raise Exception('{0} social provider missing settings.'.format(self.type.title()))

        self.initialize_settings = self.settings.get('init_params', {}).copy()
        self.initialize_settings.update(kwargs)

        self.site_root = SIMPLE_SOCIALAUTH_SITEROOT
        if not self.site_root:
            self.site_root = "{0}://{1}".format('https' if SIMPLE_SOCIALAUTH_SECURE else 'http', Site.objects.get_current().domain)

        callback_uri = '{0}{1}'.format(
            self.site_root,
            reverse('simple_socialauth-login_callback', args=(self.type,))
        )

        if oauth1:
            self.session = OAuth1Session(
                callback_uri=callback_uri,
                **self.initialize_settings
            )
        else:
            self.session = OAuth2Session(
                redirect_uri=callback_uri,
                **self.initialize_settings
            )

    def connect(self, **kwargs):
        authorization_url = getattr(self, 'authorization_url', None)
        if not authorization_url:
            raise Exception(
                'no authorization_url set for provider: '.format(self.type)
            )

        self.authorize_params = self.settings.get('authorize_params', {}).copy()
        self.authorize_params.update(kwargs)

        if self.oauth1:
            request_token_url = getattr(self, 'request_token_url', None)
            if not request_token_url:
                raise Exception(
                    'no request_token_url set for provider: '.format(self.type)
                )

            self.session.fetch_request_token(
                request_token_url,
                **self.authorize_params
            )
            return self.session.authorization_url(authorization_url)
        else:
            url, state = self.session.authorization_url(
                authorization_url,
                **self.authorize_params
            )
            return url

    def callback(self, oauth_data, **kwargs):
        access_token_url = getattr(self, 'access_token_url', None)
        if not access_token_url:
            raise Exception(
                'no access_token_url set for provider: '.format(self.type)
            )

        self.callback_params = self.settings.get('callback_params', {}).copy()
        self.callback_params.update(kwargs)

        if self.oauth1:
            self.session.parse_authorization_response(oauth_data)
            return self.session.fetch_access_token(
                access_token_url,
                **self.callback_params
            )
        else:
            return self.session.fetch_token(
                access_token_url,
                authorization_response=oauth_data,
                **self.callback_params
            )

    def get_social_user_info(self):
        url = self.user_api_url + (self.settings.get('user_api_url_params') or '')

        r = self.session.get(url)
        if r.status_code == 200:
            return r.json()

        return {}





from __future__ import unicode_literals
from django.urls import re_path

from . import views


urlpatterns = [
    re_path(r'^(?P<provider_type>[\w\-]+)/login/$', views.LoginView.as_view(), name='simple_socialauth-login'),
    re_path(r'^(?P<provider_type>[\w\-]+)/callback/$', views.CallbackView.as_view(), name='simple_socialauth-login_callback'),
    re_path(r'^(?P<provider_type>[\w\-]+)/complete/$', views.CompleteView.as_view(), name='simple_socialauth-login_complete'),
]



import logging

from ..settings import SIMPLE_SOCIALAUTH_PROVIDERS

logger = logging.getLogger(__name__)


def import_class(item):
    try:
        module_name, klass_name = item.rsplit('.', 1)
        mod = __import__(module_name, fromlist=module_name)
        return getattr(mod, klass_name)
    except:
        pass


class ProviderRegistry(object):
    providers = {}
    TYPE_CHOICES = []

    def __init__(self):
        for provider_ in SIMPLE_SOCIALAUTH_PROVIDERS:
            provider_class = import_class(provider_)
            if provider_class:
                logger.debug('{0} social provider loaded successfully.'.format(provider_class.type))
                provider_type = provider_class.type
                self.providers[provider_type] = provider_class

                setattr(self, 'TYPE_{0}'.format(provider_type.upper()), provider_type)
                self.TYPE_CHOICES.append((provider_type, provider_class.display_name()))
            else:
                logger.debug('Social provider class not found/imported: {0}'.format(provider_))

        self.TYPE_CHOICES = sorted(self.TYPE_CHOICES)

    def get(self, provider_type, **kwargs):
        provider_class = self.providers.get(provider_type)
        if provider_class:
            return provider_class(**kwargs)


registry = ProviderRegistry()



from __future__ import unicode_literals
from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend as StandardModelBackend

from .models import SocialAccount

UserModel = get_user_model()


class Backend(StandardModelBackend):
    def authenticate(self, request, social=False, **kwargs):
        if social:
            uid = kwargs.get('uid', '')
            provider = kwargs.get('provider', '')
            if uid and provider:
                try:
                    return SocialAccount.objects.select_related('user').get(
                        uid=uid, provider=provider
                    ).user
                except SocialAccount.DoesNotExist:
                    pass

            pk = kwargs.get('pk')
            if pk:
                try:
                    user = UserModel.objects.get(pk=pk)
                except UserModel.DoesNotExist:
                    # Run the default password hasher once to reduce the timing
                    # difference between an existing and a nonexistent user (#20760).
                    UserModel().set_password(None)
                else:
                    # `user_can_authenticate` only appeared in Django 1.10, so
                    # ignore it if it doesn't exist for lower versions' sakes
                    if not hasattr(self, 'user_can_authenticate') or self.user_can_authenticate(user):
                        return user



from __future__ import unicode_literals
from requests_oauthlib.compliance_fixes import linkedin_compliance_fix

from .base import BaseProvider


# access tokens: 60 day expiry
class LinkedinProvider(BaseProvider):
    type = 'linkedin'
    name = 'LinkedIn'

    def __init__(self, **kwargs):
        self.authorization_url = 'https://www.linkedin.com/uas/oauth2/authorization'
        self.access_token_url = 'https://www.linkedin.com/uas/oauth2/accessToken'
        self.user_api_url = 'https://api.linkedin.com/v1/people/~'
        super(LinkedinProvider, self).__init__(**kwargs)
        linkedin_compliance_fix(self.session)
        self.session.headers['x-li-format'] = 'json'

    def get_social_user_info(self):
        data = super(LinkedinProvider, self).get_social_user_info()
        if data:
            uid = data.get('id')
            first_name = data.get('firstName', '')
            last_name = data.get('lastName', '')
            name = "{0} {1}".format(first_name.strip(), last_name.strip()).strip()

            return {
                'source_data': data,
                'uid': uid,
                'username': uid,
                'name': name,
                'first_name': first_name,
                'last_name': last_name
            }
        return {}



from __future__ import unicode_literals
from django import forms
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

from .settings import SIMPLE_SOCIALAUTH_GENERATE_USERNAME

User = get_user_model()


class UserCreateCleanMixin(object):
    def clean_email(self):
        instance = getattr(self, 'instance')
        email = self.cleaned_data.get('email', '').lower().strip()

        qs = User.objects.filter(email__iexact=email)
        if instance:
            qs = qs.exclude(pk=instance.pk)

        if qs.exists():
            raise forms.ValidationError(
                self.error_messages['email_unique'],
                code='email_unique'
            )

        return email

    def clean_username(self):
        instance = getattr(self, 'instance')
        username = self.cleaned_data.get('username', '').strip()

        qs = User.objects.filter(username__iexact=username)
        if instance:
            qs = qs.exclude(pk=instance.pk)

        if qs.exists():
            raise forms.ValidationError(
                self.error_messages['username_unique'],
                code='username_unique'
            )

        return username


def create_model_form():
    class _PostSocialSignupForm(UserCreateCleanMixin, forms.ModelForm):
        error_messages = {
            'email_unique': _("A user with that email address already exists."),
            'username_unique': _("A user with that username already exists.")
        }

        class Meta:
            model = User
            fields = ('username', 'email')

    # only process fields which exist in the User model; if a custom
    # User model is defined without a `username` field, this lets us
    # still use this modelform
    # also, if SIMPLE_SOCIALAUTH_GENERATE_USERNAME is True, we don't
    # want this form to do anything with `username` (we'll generate it)
    available_model_fields = [f.name for f in User._meta.get_fields() if f.name != 'username' or not SIMPLE_SOCIALAUTH_GENERATE_USERNAME]
    _PostSocialSignupForm.Meta.fields = [f for f in _PostSocialSignupForm.Meta.fields if f in available_model_fields]
    return type('PostSocialSignupForm', (_PostSocialSignupForm,), {})

# This moderately terrible way of creating the modelform is needed to
# account for the possibility that developers may have a custom model
# that does not include a `username` field - the assumption IS made
# however that there is always an `email` field.
PostSocialSignupForm = create_model_form()


from __future__ import unicode_literals

from .base import BaseProvider


# access tokens: no expiry
class TwitterProvider(BaseProvider):
    type = 'twitter'

    def __init__(self, **kwargs):
        # use authenticate instead of authorize as the latter forces the user
        # to accept permissions each time as per the oauth spec (?)
        # self.authorization_url = 'https://api.twitter.com/oauth/authorize'
        self.authorization_url = 'https://api.twitter.com/oauth/authenticate'
        self.request_token_url = 'https://api.twitter.com/oauth/request_token'
        self.access_token_url = 'https://api.twitter.com/oauth/access_token'
        self.user_api_url = 'https://api.twitter.com/1.1/account/verify_credentials.json'
        super(TwitterProvider, self).__init__(True,  **kwargs)

    def get_social_user_info(self):
        data = super(TwitterProvider, self).get_social_user_info()
        if data:
            name = data.get('name', '')
            name_split = name.split(' ', 1)
            return {
                'source_data': data,
                'uid': data.get('id'),
                'username': data.get('screen_name', ''),
                'name': name,
                'first_name': name_split[0],
                'last_name': name_split[1] if len(name_split) > 1 else ''
            }
        return {}


from __future__ import unicode_literals

from .base import BaseProvider


# access tokens: 60 day expiry
class GithubProvider(BaseProvider):
    type = 'github'

    def __init__(self, **kwargs):
        self.authorization_url = 'https://github.com/login/oauth/authorize'
        self.access_token_url = 'https://github.com/login/oauth/access_token'
        self.user_api_url = 'https://api.github.com/user'
        super(GithubProvider, self).__init__(**kwargs)

    def get_social_user_info(self):
        data = super(GithubProvider, self).get_social_user_info()
        if data:
            uid = data.get('id')
            name = data.get('name', '')
            name_split = name.split(' ', 1)
            return {
                'source_data': data,
                'uid': uid,
                'username': data.get('login', ''),
                'email': data.get('email', ''),
                'name': name,
                'first_name': name_split[0],
                'last_name': name_split[1] if len(name_split) > 1 else ''
            }
        return {}


from __future__ import unicode_literals
from django.conf import settings

SIMPLE_SOCIALAUTH_LOGIN_SUCCESS_REDIRECT_URL = getattr(settings, 'SIMPLE_SOCIALAUTH_LOGIN_SUCCESS_REDIRECT_URL', None) or getattr(settings, 'LOGIN_REDIRECT_URL', '/')
SIMPLE_SOCIALAUTH_LOGIN_ERROR_REDIRECT_URL = getattr(settings, 'SIMPLE_SOCIALAUTH_LOGIN_ERROR_REDIRECT_URL', '/')
SIMPLE_SOCIALAUTH_PROVIDERS = getattr(settings, 'SIMPLE_SOCIALAUTH_PROVIDERS', ())
SIMPLE_SOCIALAUTH_PROVIDERS_SETTINGS = getattr(settings, 'SIMPLE_SOCIALAUTH_PROVIDERS_SETTINGS', {})
SIMPLE_SOCIALAUTH_SITEROOT = getattr(settings, 'SIMPLE_SOCIALAUTH_SITEROOT', '')
SIMPLE_SOCIALAUTH_SECURE = getattr(settings, 'SIMPLE_SOCIALAUTH_SECURE', True)
SIMPLE_SOCIALAUTH_GENERATE_USERNAME = getattr(settings, 'SIMPLE_SOCIALAUTH_GENERATE_USERNAME', False)

""" Example data for currently supported providers - put something like this in your Django project's settings.py:

SIMPLE_SOCIALAUTH_PROVIDERS_SETTINGS = {
    'twitter': {
        'init_params': {
            'client_key': 'my-key',
            'client_secret': 'my-secret'
        },
        'authorize_params': {},
        'callback_params': {}
    },
    'facebook': {
        'init_params': {
            'client_id': 'my-client_id',
            # 'scope': ['manage_pages', 'publish_stream']
        },
        'authorize_params': {},
        'callback_params': {
            'client_secret': 'my-client_secret'
        }
        'user_api_url_params': "?fields=id,first_name,last_name,name,email,age_range,gender,birthday,languages,hometown"
    },
    'pinterest': {
        'init_params': {
            'client_id': 'my-client_id',
        },
        'authorize_params': {},
        'callback_params': {
            'client_secret': 'my-client_secret'
        }
        'user_api_url_params': "?fields=id,username,first_name,last_name"
    },
    'google': {
        'init_params': {
            'client_id': 'my-client_id',
            'auto_refresh_url': 'https://accounts.google.com/o/oauth2/token',
            # see https://www.googleapis.com/discovery/v1/apis/oauth2/v2/rest?fields=auth(oauth2(scopes))
            'scope': ['https://www.googleapis.com/auth/plus.login']
        },
        'authorize_params': {
            'access_type': 'offline'
        },
        'callback_params': {
            'client_secret': 'my-client_secret'
        }
    },
    'github': {
        'init_params': {
            'client_id': 'my-client_id',
            # 'scope': ['manage_pages', 'publish_stream']
        },
        'authorize_params': {},
        'callback_params': {
            'client_secret': 'my-client_secret'
        }
    },
    'angellist': {
        'init_params': {
            'client_id': 'my-client_id'
        },
        'callback_params': {
            'client_secret': 'my-client_secret'
        }
    },
    'linkedin': {
        'init_params': {
            'client_id': 'my-client_id'
        },
        'callback_params': {
            'client_secret': 'my-client_secret'
        }
        'user_api_url_params': ":(id,first-name,last-name)"
    }
}
"""



from __future__ import unicode_literals
from django import dispatch

login = dispatch.Signal(providing_args=["request", "account", "user_info", "created"])
connect = dispatch.Signal(providing_args=["request", "account", "user_info", "created"])



from __future__ import unicode_literals
import datetime
import json

from django.conf import settings
from django.db import models
from django.utils import timezone
from six import python_2_unicode_compatible

from .providers import registry


@python_2_unicode_compatible
class SocialAccount(models.Model):
    # TODO: Does all the token related data need to move to a separate
    # 'SocialToken' model to allow for 1:many relationship with
    # SocialAccount (multiple logins/devices, etc)?

    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='social_accounts', on_delete=models.CASCADE)
    provider = models.CharField(max_length=32)
    uid = models.CharField(max_length=256)

    access_token = models.TextField(default='', blank=True)
    secret = models.TextField(default='', blank=True)  # OAuth1 only
    expires = models.DateTimeField(null=True, blank=True)

    _data = models.TextField(blank=True, default="{}", help_text="Original token response from the OAuth provider.")

    class Meta:
        unique_together = ('uid', 'provider')

    def __str__(self):
        return "{}: {}".format(self.provider, self.uid)

    @property
    def data(self):
        if not hasattr(self, '_data_loaded'):
            self._data_loaded = json.loads(self._data)
        return self._data_loaded

    @data.setter
    def data(self, value):
        self._data_loaded = value
        self._data = json.dumps(value)

    @property
    def session(self):
        if not hasattr(self, '_provider'):
            if self.secret:
                # OAuth1-based
                kw = {
                    'resource_owner_key': self.access_token,
                    'resource_owner_secret': self.secret
                }
            else:
                # OAuth2-based
                if self.data:
                    # NOTE: auto-refreshing the URL ought to just 'work'
                    # if all the arguments are passed in properly during
                    # initialization
                    kw = {'token': self.data}
                else:
                    kw = {
                        'token': {
                            'token_type': 'Bearer',
                            'access_token': self.access_token
                        }
                    }
            self._provider = registry.get(self.provider, **kw)

        return self._provider.session

    @classmethod
    def create_or_update(cls, user, user_info, token_data):
        uid = user_info.get('uid')
        provider = user_info.get('provider')

        # OAUTH2: if expires_in is in the token data
        expires_in = token_data.get('expires_in')
        # OAUTH1: if secret is set in the token data
        secret = token_data.get('oauth_token_secret', '')

        access_token = token_data.get('access_token', '')
        if secret:
            # if OAUTH1, get the real token string
            access_token = token_data.get('oauth_token', '')

        if uid and provider and access_token:
            token_data.pop('info', None)
            return cls.objects.update_or_create(
                uid=uid,
                provider=provider,
                defaults=dict(
                    user=user,
                    access_token=access_token,
                    secret=secret,
                    expires=(timezone.now() + datetime.timedelta(seconds=int(expires_in))) if expires_in else None,
                    data=token_data
                )
            )

        return None, False



from __future__ import unicode_literals
from setuptools import setup, find_packages

import simple_socialauth

setup(
    name='django-simple-socialauth',
    version=simple_socialauth.__version__,
    description="Django social account authentication app based on requests-oauthlib.",
    long_description='View `django-simple-socialauth documentation on Github  <https://github.com/jaddison/django-simple-socialauth>`_.',
    keywords='django social authentication oauth register login auth',

    url='https://github.com/jaddison/django-simple-socialauth',

    author='jaddison',
    author_email='addi00+github.com@gmail.com',

    license='MIT',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Internet :: WWW/HTTP',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=['requests-oauthlib'],
)



from __future__ import unicode_literals
from requests_oauthlib.compliance_fixes import facebook_compliance_fix

from .base import BaseProvider


# access tokens: 60 day expiry
class FacebookProvider(BaseProvider):
    type = 'facebook'

    def __init__(self, **kwargs):
        self.authorization_url = 'https://www.facebook.com/dialog/oauth'
        self.access_token_url = 'https://graph.facebook.com/oauth/access_token'
        self.user_api_url = 'https://graph.facebook.com/me'
        super(FacebookProvider, self).__init__(**kwargs)
        facebook_compliance_fix(self.session)

    def get_social_user_info(self):
        data = super(FacebookProvider, self).get_social_user_info()
        if data:
            uid = data.get('id')
            return {
                'source_data': data,
                'uid': uid,
                'email': data.get('email', ''),
                'name': data.get('name', ''),
                'first_name': data.get('first_name', ''),
                'last_name': data.get('last_name', ''),
            }
        return {}



from __future__ import unicode_literals
from django.contrib import admin

from .models import SocialAccount


class SocialAccountAdmin(admin.ModelAdmin):
    list_display = ('uid', 'provider', 'user', 'expires')
    list_filter = ('provider', 'expires',)
    search_fields = ('user__email', 'user__id', 'user__username', 'access_token')
    readonly_fields = ('provider', 'uid', 'user')

admin.site.register(SocialAccount, SocialAccountAdmin)
